package Net::Async::UWSGI;
# ABSTRACT: IO::Async support for dealing with UWSGI requests
use strict;
use warnings;

our $VERSION = '0.006';

=head1 NAME

Net::Async::UWSGI - support the universal webserver gateway interface protocol in IO::Async

=head1 VERSION

version 0.006

=head1 DESCRIPTION

See L<Net::Async::UWSGI::Server>.

=cut

1;

__END__

=head1 AUTHOR

Tom Molesworth <cpan@perlsite.co.uk>

=head1 LICENSE

Copyright Tom Molesworth 2013-2015. Licensed under the same terms as Perl itself.
